@@ -19,7 +19,7 @@
  * Boston, MA 02110-1301 USA
  *
  * Authors:
- * 	Arjan van de Ven <arjan@linux.intel.com>
+ *	Arjan van de Ven <arjan@linux.intel.com>
  */
 
 #include <unistd.h>
@@ -46,7 +46,7 @@ char cpufreqstrings[6][80];
 int topfreq = -1;
 
 static void zap(void)
-{	
+{
 	memset(freqs, 0, sizeof(freqs));
 }
 
@@ -54,16 +54,27 @@ int sort_by_count (const void *av, const void *bv)
 {
         const struct cpufreqdata       *a = av, *b = bv;
         return b->count - a->count;
-} 
+}
 
 int sort_by_freq (const void *av, const void *bv)
 {
         const struct cpufreqdata       *a = av, *b = bv;
         return b->frequency - a->frequency;
-} 
+}
+
+
+static unsigned long turbo_hz;
+
+static int is_turbo(unsigned long hz)
+{
+	if (hz == turbo_hz)
+		return 1;
+	return 0;
+}
+
 
 static char *HzToHuman(unsigned long hz)
-{	
+{
 	static char buffer[1024];
 	memset(buffer, 0, 1024);
 	unsigned long long Hz;
@@ -79,6 +90,9 @@ static char *HzToHuman(unsigned long hz)
 	if (Hz>1500000)
 		sprintf(buffer, _("%6.2f Ghz"), (Hz+5000.0)/1000000);
 
+	if (is_turbo(hz))
+		sprintf(buffer, "%s", _("Turbo Mode"));
+
 
 	return buffer;
 }
@@ -164,6 +178,9 @@ void  do_cpufreq_stats(void)
 		maxfreq=4;
 	qsort(&delta, maxfreq+1, sizeof(struct cpufreqdata), sort_by_freq);
 
+	if (delta[0].frequency == delta[1].frequency + 1000)
+		turbo_hz = delta[0].frequency;
+
 	topfreq = -1;
 	for (ret = 0 ; ret<=maxfreq; ret++) {
 		sprintf(cpufreqstrings[ret+1], "%6s   %5.1f%%\n", HzToHuman(delta[ret].frequency), delta[ret].count * 100.0 / total_time);
