@@ -19,7 +19,7 @@
  * Boston, MA 02110-1301 USA
  *
  * Authors:
- * 	Arjan van de Ven <arjan@linux.intel.com>
+ *	Arjan van de Ven <arjan@linux.intel.com>
  */
 
 #include <unistd.h>
@@ -32,56 +32,6 @@
 
 #include "powertop.h"
 
-void set_laptop_mode(void)
-{
-	FILE *file;
-	file = fopen("/proc/sys/vm/laptop_mode", "w");
-	if (!file)
-		return;
-	fprintf(file,"5\n");
-	fclose(file);
-}
-
-void suggest_laptop_mode(void)
-{
-	FILE *file;
-	int i;
-	char buffer[1024];
-	/*
-	 * Check to see if we are on AC - lots of distros have
-	 * annoying scripts to turn laptop mode off when on AC, which
-	 * results in annoying distracting return of set laptop mode
-	 * hint.
-	 */
-	file = fopen("/proc/acpi/ac_adapter/AC/state", "r");
-	if (!file)
-		return;
-	memset(buffer, 0, 1024);
-	if (!fgets(buffer, 1023, file)) {
-		fclose(file);
-		return;
-	}
-	fclose(file);
-	if (strstr(buffer, "on-line") != NULL)
-		return;
-
-	/* Now check for laptop mode */
-	file = fopen("/proc/sys/vm/laptop_mode", "r");
-	if (!file)
-		return;
-	memset(buffer, 0, 1024);
-	if (!fgets(buffer, 1023, file)) {
-		fclose(file);
-		return;
-	}
-	i = strtoul(buffer, NULL, 10);
-	if (i<1) {
-		add_suggestion( _("Suggestion: Enable laptop-mode by executing the following command:\n"
-		 	"   echo 5 > /proc/sys/vm/laptop_mode \n"), 15, 'L', _(" L - enable Laptop mode "), set_laptop_mode);
-	}
-	fclose(file);
-}
-
 void nmi_watchdog_off(void)
 {
 	FILE *file;
@@ -107,7 +57,7 @@ void suggest_nmi_watchdog(void)
 	i = strtoul(buffer, NULL, 10);
 	if (i!=0) {
 		add_suggestion( _("Suggestion: disable the NMI watchdog by executing the following command:\n"
-		 	"   echo 0 > /proc/sys/kernel/nmi_watchdog \n"
+			"   echo 0 > /proc/sys/kernel/nmi_watchdog \n"
 			"The NMI watchdog is a kernel debug mechanism to detect deadlocks"), 25, 'N', _(" N - Turn NMI watchdog off "), nmi_watchdog_off);
 	}
 	fclose(file);
@@ -121,12 +71,12 @@ void suggest_hpet(void)
 	if (!file)
 		return;
 	memset(buffer, 0, 1024);
-	
+
 	if (!fgets(buffer, 1023, file)) {
 		fclose(file);
 		return;
 	}
-	
+
 	if (strstr(buffer, "hpet")) {
 		fclose(file);
 		return;
@@ -149,6 +99,7 @@ void ac97_power_on(void)
 	fclose(file);
 	if (access("/dev/dsp", F_OK))
 		return;
+	/* kick power mgmt update to the driver */
 	file = fopen("/dev/dsp", "w");
 	if (file) {
 		fprintf(file,"1");
@@ -170,12 +121,52 @@ void suggest_ac97_powersave(void)
 	}
 	if (buffer[0]=='N') {
 		add_suggestion( _("Suggestion: enable AC97 powersave mode by executing the following command:\n"
-		 	"   echo 1 > /sys/module/snd_ac97_codec/parameters/power_save \n"
+			"   echo 1 > /sys/module/snd_ac97_codec/parameters/power_save \n"
 			"or by passing power_save=1 as module parameter."), 25, 'A', _(" A - Turn AC97 powersave on "), ac97_power_on);
 	}
 	fclose(file);
 }
 
+void hda_power_on(void)
+{
+	FILE *file;
+
+	file = fopen("/sys/module/snd_hda_intel/parameters/power_save", "w");
+	if (!file)
+		return;
+	fprintf(file,"1");
+	fclose(file);
+	if (access("/dev/dsp", F_OK))
+		return;
+	/* kick power mgmt update to the driver */
+	file = fopen("/dev/dsp", "w");
+	if (file) {
+		fprintf(file,"1");
+		fclose(file);
+	}
+}
+
+void suggest_hda_powersave(void)
+{
+	FILE *file;
+	char buffer[1024];
+
+	file = fopen("/sys/module/snd_hda_intel/parameters/power_save", "r");
+	if (!file)
+		return;
+	memset(buffer, 0, 1024);
+	if (!fgets(buffer, 1023, file)) {
+		fclose(file);
+		return;
+	}
+	if (buffer[0]=='0') {
+		add_suggestion( _("Suggestion: enable HD audio powersave mode by executing the following command:\n"
+			"   echo 1 > /sys/module/snd_hda_intel/parameters/power_save \n"
+			"or by passing power_save=1 as module parameter."), 25, 'A', _(" A - Turn HD audio powersave on "), hda_power_on);
+	}
+	fclose(file);
+}
+
 void noatime_on(void)
 {
 	system("/bin/mount -o remount,noatime,nodiratime /");
@@ -199,7 +190,7 @@ void suggest_noatime(void)
 	}
 	if (suggest) {
 		add_suggestion( _("Suggestion: enable the noatime filesystem option by executing the following command:\n"
-		 	"   mount -o remount,noatime /          or by pressing the T key \n"
+			"   mount -o remount,noatime /          or by pressing the T key \n"
 			"noatime disables persistent access time of file accesses, which causes lots of disk IO."), 5, 'T', _(" T - enable noatime "), noatime_on);
 	}
 	fclose(file);
@@ -221,7 +212,7 @@ void suggest_powersched(void)
 	char buffer[1024];
 	int suggest = 0;
 	int cpu;
-	
+
 	file = fopen("/sys/devices/system/cpu/sched_mc_power_savings","r");
 	if (!file)
 		return;
@@ -245,12 +236,11 @@ void suggest_powersched(void)
 		return;
 	while (!feof(file)) {
 		memset(buffer, 0, 1024);
-		char *c;
 		if (!fgets(buffer, 1023, file))
 			break;
 		if (strstr(buffer, "cpu cores")) {
-			c = strchr(buffer, ':');
-			if (!c) 
+			char *c = strchr(buffer, ':');
+			if (!c)
 				continue;
 			c++;
 			if (strtoll(c, NULL, 10) >= cpu)
@@ -262,7 +252,7 @@ void suggest_powersched(void)
 
 	if (suggest) {
 		add_suggestion( _("Suggestion: enable the power aware CPU scheduler with the following command:\n"
-		 	"  echo 1 > /sys/devices/system/cpu/sched_mc_power_savings\n"
+			"  echo 1 > /sys/devices/system/cpu/sched_mc_power_savings\n"
 			"or by pressing the C key."), 5, 'C', _(" C - Power aware CPU scheduler "), powersched_on);
 	}
 }
@@ -295,7 +285,7 @@ void suggest_writeback_time(void)
 	if (i<1400) {
 		char line[1024];
 		sprintf(line,_("Suggestion: increase the VM dirty writeback time from %1.2f to 15 seconds with:\n"
-			 	"  echo 1500 > /proc/sys/vm/dirty_writeback_centisecs \n"
+				"  echo 1500 > /proc/sys/vm/dirty_writeback_centisecs \n"
 				"This wakes the disk up less frequently for background VM activity"),
 			i/100.0);
 		add_suggestion(line, 15, 'W', _(" W - Increase Writeback time "), writeback_long);
